update study_groups set formid=1;

ALTER TABLE public.study_groups ALTER COLUMN formid SET NOT NULL;
ALTER TABLE public.study_groups ALTER COLUMN formid SET DEFAULT 1;

ALTER TABLE public.study_groups DROP CONSTRAINT study_groups_facultyid_gradeid_groupnum_key;
ALTER TABLE ONLY study_groups
  ADD CONSTRAINT study_groups_facultyid_gradeid_groupnum_formid_key UNIQUE (facultyid, gradeid, groupnum, formid);

ALTER TABLE ONLY study_groups
  ADD CONSTRAINT study_groups_ibfk_4 FOREIGN KEY (formid) REFERENCES study_form(id);

DROP FUNCTION IF EXISTS public.creategroup(pgradeid integer, pgroupnum integer, pspecname character varying, pfacultyid integer, pyear integer);
CREATE OR REPLACE FUNCTION public.creategroup(pgradeid integer, pgroupnum integer, pspecname character varying, pfacultyid integer, pyear integer, pformid integer)
  RETURNS integer
LANGUAGE plpgsql
AS $function$
DECLARE vGroupID INT DEFAULT null;
        vSpecId INT DEFAULT null;
        vGroupYear INT DEFAULT null;
        vIsSpecMatch BOOL DEFAULT null;

BEGIN
  --select public.creategroup(
  --	:pgradeid,	-- put the pgradeid parameter value instead of 'pgradeid' (int4)
  --	:pgroupnum,	-- put the pgroupnum parameter value instead of 'pgroupnum' (int4)
  --	:pspecname,	-- put the pspecname parameter value instead of 'pspecname' (varchar)
  --	:pfacultyid,	-- put the pfacultyid parameter value instead of 'pfacultyid' (int4)
  --	:pyear 	-- put the pyear parameter value instead of 'pyear' (int4)
  --);
  -- create specialization vSpecId - UNIQUE (pSpecName,pFacultyID)
  select specializations.id into vSpecId
  from specializations
  where specializations.facultyid=pFacultyID and specializations."name" like pSpecName;
  if (vSpecId is null) then
    INSERT INTO specializations (Name, Abbr, FacultyID)
    VALUES  (pSpecName, NULL, pFacultyID)
    returning id into vSpecId;
  end if;


  -- create group  vGroupID - UNIQUE (pGradeID, pGroupNum, pFacultyID)
  select study_groups.id into vGroupID
  from study_groups
  where study_groups.gradeid=pgradeID
        and study_groups.groupnum= pGroupNum
        and study_groups.facultyid=pFacultyID
        and study_groups.formid=pFormID;
  if ( vGroupID  is null) then
    INSERT INTO study_groups (GradeID, GroupNum, FacultyID, FormID)
    VALUES (pGradeID, pGroupNum, pFacultyID, pFormID)
    returning id into  vGroupID ;
  end if;


  SELECT groups_years.groupid, groups_years.specializationid = vSpecId
  INTO vGroupYear, vIsSpecMatch
  FROM groups_years
  WHERE groups_years.groupid = vGroupID AND groups_years."year" = pYear
  LIMIT 1;

  IF  (vGroupYear is null) THEN
    INSERT INTO groups_years (GroupID, Year, SpecializationID)
    VALUES (vGroupID, pYear, vSpecId);
  ELSEIF NOT vIsSpecMatch THEN
    RETURN -1;
  END IF;

  RETURN vGroupID;
  EXCEPTION
  when others then
    RETURN -1;
END
$function$;

DROP FUNCTION IF EXISTS public.getgroupsfordisciplineall(pdisciplineid integer);
CREATE OR REPLACE FUNCTION public.getgroupsfordisciplineall(pdisciplineid integer)
  RETURNS TABLE("ID" integer, "GroupNum" integer, "GradeID" integer, "GradeNum" integer, "Degree" bachelor_master_specialist, "SpecID" integer, "SpecName" character varying, "SpecAbbr" character varying, "FormID" character varying)
LANGUAGE plpgsql
AS $function$

DECLARE vSemesterID INT DEFAULT -1; vYear INT DEFAULT -1;
BEGIN
  vSemesterID := GetDisciplineProperty(pDisciplineID, 'semester');
  SELECT "year"  into vYear from semesters where semesters.ID = vSemesterID;

  -- general + attached
  CREATE TEMPORARY TABLE IF NOT EXISTS tGroup AS (
    SELECT students_groups.GroupID
    FROM view_disciplines_recordbooks
      INNER JOIN students_groups ON students_groups.RecordBookID = view_disciplines_recordbooks.RecordBookID AND
                                    students_groups.SemesterID = vSemesterID AND
                                    students_groups.State <= 'outlet'
    WHERE view_disciplines_recordbooks.DisciplineID = pDisciplineID AND
          'attach' = COALESCE(view_disciplines_recordbooks.Type, 'attach')
    GROUP BY students_groups.GroupID
  );

  return query
  SELECT  view_groups.GroupID AS "ID",
          view_groups.GroupNum as "GroupNum",
          view_groups.GradeID	as "GradeID",
          view_groups.GradeNum as "GradeNum",
          view_groups."degree" as "Degree",
          view_groups.SpecID as "SpecID",
          view_groups.SpecName as "SpecName",
          view_groups.SpecAbbr as "SpecAbbr",
          (SELECT study_form.FormName from study_form WHERE study_form.id = view_groups.FormID)
          --view_groups.FormID as "FormID"
  FROM tGroup
    INNER JOIN view_groups ON tGroup.GroupID = view_groups.GroupID AND view_groups.Year = vYear
  ORDER BY view_groups.GradeID ASC, view_groups.GroupID ASC;
END;
$function$;

DROP VIEW public.view_groups;
CREATE OR REPLACE VIEW public.view_groups
  AS SELECT study_groups.id AS groupid,
       study_groups.groupnum,
            groups_years.name AS groupname,
            grades.id AS gradeid,
            grades.num AS gradenum,
            grades.degree,
            specializations.id AS specid,
            specializations.name AS specname,
            specializations.abbr AS specabbr,
            specializations.code AS speccode,
            study_groups.formid AS formid,
            faculties.id AS facultyid,
            faculties.name AS facultyname,
            faculties.abbr AS facultyabbr,
            groups_years.year
     FROM groups_years
       JOIN study_groups ON groups_years.groupid = study_groups.id
       JOIN specializations ON groups_years.specializationid = specializations.id
       JOIN grades ON study_groups.gradeid = grades.id
       JOIN faculties ON faculties.id = specializations.facultyid;